﻿//------------------------------------------------------------------------------
// <copyright file="MacroCommand.cs" company="Company">
//     Copyright (c) Company.  All rights reserved.
// </copyright>
//------------------------------------------------------------------------------

using System;
using System.ComponentModel.Design;
using System.Globalization;
using Microsoft.VisualStudio.Shell;
using Microsoft.VisualStudio.Shell.Interop;
using EnvDTE80;
using EnvDTE;

namespace VSMacro
{
    internal sealed class MacroCommand
    {
        public const int CommandId = 0x3002; // cmdidMyDynamicStartCommand in .vsct
        public static readonly Guid CommandSet = new Guid("6d15ea04-448b-4c65-b70c-cbfaa3c105b7"); // guidMacroCommandPackageCmdSet in .vsct
        private readonly Package package;

        private DTE2 dte2;
        MacroRepository _repo;

        private MacroCommand(Package package)
        {
            if (package == null)
            {
                throw new ArgumentNullException("package");
            }

            _repo = new MacroRepository();
            
            this.package = package;

            OleMenuCommandService commandService = this.ServiceProvider.GetService(typeof(IMenuCommandService)) as OleMenuCommandService;
            if (commandService != null)
            {
                CommandID dynamicItemRootId = new CommandID(CommandSet, (int)CommandId);
                DynamicItemMenuCommand dynamicMenuCommand = new DynamicItemMenuCommand(dynamicItemRootId,
                    _repo,
                    OnInvokedDynamicItem,
                    OnBeforeQueryStatusDynamicItem);

                commandService.AddCommand(dynamicMenuCommand);
            }

            dte2 = (DTE2)this.ServiceProvider.GetService(typeof(DTE));
        }

        private void OnBeforeQueryStatusDynamicItem(object sender, EventArgs args)
        {
            DynamicItemMenuCommand matchedCommand = (DynamicItemMenuCommand)sender;

            matchedCommand.Enabled = true;
            matchedCommand.Visible = true;

            matchedCommand.Text = _repo.GetCommandTextByPos(matchedCommand.MatchedCommandId);
            matchedCommand.MatchedCommandId = 0;
        }

        private void OnInvokedDynamicItem(object sender, EventArgs args)
        {
            DynamicItemMenuCommand invokedCommand = (DynamicItemMenuCommand)sender;

            string srcCode = _repo.GetCommandSourceCodeByPos(invokedCommand.MatchedCommandId);

            VsShellUtilities.ShowMessageBox(
                this.ServiceProvider,
                srcCode,
                "MyTitle",
                OLEMSGICON.OLEMSGICON_INFO,
                OLEMSGBUTTON.OLEMSGBUTTON_OK,
                OLEMSGDEFBUTTON.OLEMSGDEFBUTTON_FIRST);
        }

        public static MacroCommand Instance
        {
            get;
            private set;
        }

        private IServiceProvider ServiceProvider
        {
            get
            {
                return this.package;
            }
        }

        public static void Initialize(Package package)
        {
            Instance = new MacroCommand(package);
        }
    }
}
